@@ -10,6 +10,7 @@ use Mojo::Base -base;
 use Mojo::ByteStream;
 use Mojo::Util;
 use Mojolicious::Types;
+use Scalar::Util 'blessed';
 
 # this may change in future version
 use constant DEFAULT_VIDEO_HEIGHT => 390;
@@ -21,6 +22,12 @@ use constant DEFAULT_VIDEO_WIDTH  => 640;
 
 Returns the part of the URL identifying the media. Default is empty string.
 
+=head2 provider_name
+
+  $str = $self->provider_name;
+
+Example: "Twitter".
+
 =head2 ua
 
 Holds a L<Mojo::UserAgent> object.
@@ -32,9 +39,9 @@ Holds a L<Mojo::URL> object.
 =cut
 
 has media_id => '';
-has ua => sub { die "Required in constructor" };
-
-sub url { shift->{url} }
+sub provider_name { ucfirst shift->url->host }
+has ua  => sub { die "Required in constructor" };
+has url => sub { shift->_tx->req->url };
 
 # should this be public?
 has _tx => undef;
@@ -109,14 +116,31 @@ sub to_embed {
   qq(<a href="$url" @args>$url</a>);
 }
 
+# Mojo::JSON will automatically filter out ua and similar objects
 sub TO_JSON {
   my $self = shift;
+  my $url  = $self->url;
 
-  return {class => ref($self), url => $self->url->to_string, map { ($_ => $self->$_) } $self->_cache_attributes,};
+  return {
+    # oembed
+    # author_name => "",
+    # author_url => "",
+    # cache_age => 86400,
+    # height => $self->DEFAULT_VIDEO_HEIGHT,
+    # version => '1.0', # not really 1.0...
+    # width => $self->DEFAULT_VIDEO_WIDTH,
+    html          => $self->to_embed,
+    provider_name => $self->provider_name,
+    provider_url  => Mojo::URL->new(host => $url->host, scheme => $url->scheme),
+    type          => 'rich',
+    url           => $url,
+
+    # extra
+    pretty_url => $self->pretty_url,
+    media_id   => $self->media_id,
+  };
 }
 
-sub _cache_attributes { qw( media_id pretty_url ); }
-
 =head1 AUTHOR
 
 Jan Henning Thorsen - C<jan.henning@thorsen.pm>
