@@ -20,12 +20,16 @@ use Mojo::Base 'Mojolicious::Plugin::LinkEmbedder::Link::Text';
 
 Example C<$str>: "/username/status/123456789".
 
+=head2 provider_name
+
 =cut
 
 has media_id => sub {
   shift->url->path =~ m!^/(\w+/status/\w+)$! ? $1 : '';
 };
 
+sub provider_name {'Twitter'}
+
 =head1 METHODS
 
 =head2 to_embed
@@ -39,13 +43,12 @@ sub to_embed {
   my $media_id = $self->media_id or return $self->SUPER::to_embed;
   my %args     = @_;
 
-  $args{cards}        ||= 'hidden';
   $args{conversation} ||= 'none';
   $args{lang}         ||= 'en';
 
   return <<"HTML";
 <div class="link-embedder text-twitter">
-<blockquote class="twitter-tweet" lang="$args{lang}" data-conversation="$args{conversation}" data-cards="$args{cards}">
+<blockquote class="twitter-tweet" lang="$args{lang}" data-conversation="$args{conversation}">
 <a href="https://twitter.com/$media_id">Loading $media_id...</a>
 </blockquote>
 <script async src="//platform.twitter.com/widgets.js" charset="utf-8"></script>
